+++
title = "API Stability"
weight = 100
toc = true
aliases = [
    "/old-developer/latest/programming-guide-api-stability.html",
    "/old-developer/v2019/programming-guide-api-stability.html",
    "/old-developer/v2020/programming-guide-api-stability.html",
    "/old-developer/v2021pre/programming-guide-api-stability.html",
    "/old-developer/v2022dev0/programming-guide-api-stability.html",
    "/old-wiki/Guidelines/API_stability"
]
date = "2017-05-01"
+++

# Summary

- Define API [stability guarantees]({{< ref "#stability" >}}) for your project.
- Ensure [version numbers]({{< ref "#versioning" >}}) are changed as
  appropriate when API changes.

# API and ABI

At a high level, an API – Application Programming Interface – is the boundary
between two components when developing against them. It is closely related to
an ABI – Application Binary Interface – which is the boundary at runtime. It
defines the possible ways in which other components can interact with a
component. More concretely, this normally means the C headers of a library form
its API, and compiled library symbols its ABI. The difference between an API
and ABI is given by compilation of the code: there are certain things in a C
header, such as \#defines, which can cause a library's API to change without
changing its ABI. But these differences are mostly academic, and for all
practical purposes, API and ABI can be treated interchangeably.

Examples of API-incompatible changes to a C function would be to add a new
parameter, change the function's return type, or remove a parameter.

However, many other parts of a project can form an API. If a daemon exposes
itself on D-Bus, the interfaces exported there form an API.  Similarly, if a C
API is exposed in higher level languages by use of GIR, the GIR file forms
another API — if it changes, any higher level code using it must also change.

Other examples of more unusual APIs are configuration file locations and
formats, and GSettings schemas. Any changes to these could require code using
your library to change.

# Stability

API stability refers to some level of guarantee from a project that its API
will only change in defined ways in the future, or will not change at all.
Generally, an API is considered 'stable' if it commits to
backwards-compatibility (defined below); but APIs could also commit to being
unstable or even forwards-compatible. The purpose of API stability guarantees
is to allow people to use your project from their own code without worrying
about constantly updating their code to keep up with API changes. Typical API
stability guarantees mean that code which is compiled against one version of a
library will run without problems against all future versions of that library
with the same minor version number — or similarly that code which runs against
a daemon will continue to run against all future versions of that daemon with
the same minor version number.

It is possible to apply different levels of API stability to components within
a project. For example, the core functions in a library could be stable, and
hence their API left unchanged in future; while the newer, less core functions
could be left unstable and allowed to change wildly until the right design is
found, at which point they could be marked as stable.

Several types of stability commonly considered:

- *Unstable*: the API could change or be removed in future
- *Backwards-compatible*: only changes which permit code compiled against the
  unmodified API to continue running against the modified API are allowed (for
  example, functions cannot be removed)
- *Forwards-compatible*: only changes which permit code compiled against the
  modified API to run against the unmodified API are allowed (for example,
  functions cannot be added)
- *Totally stable*: no changes are allowed to the API, only to the
  implementation

Typically, projects commit to backwards-compatibility when they say an API is
'stable'. Very few projects commit to total stability because it would prevent
almost all further development of the project.

In some cases, old APIs may become deprecated when they are no longer needed or
do not fit with the current Apertis design. In those cases, they will be marked
as deprecated at least one release before they are removed.

# Versioning

API stability guarantees are strongly linked to project versioning; both
package versioning and libtool versioning. Libtool versioning exists entirely
for the purpose of tracking ABI stability, and is explained in detail on the
[Autotools Mythbuster](https://autotools.io/libtool/version.html) or the
[Versioning page]( {{< ref "versioning.md#libtool-versioning" >}} ).

Package versioning (*major.minor.micro*) is strongly linked to API stability:
typically, the major version number is incremented when backwards-incompatible
changes are made (for example, when functions are renamed, parameters are
changed, or functions are removed). The minor version number is incremented
when forwards-incompatible changes are made (for example, when new public API
is added). The micro version number is incremented when code changes are made
without modifying API.  See the
[Versioning page]( {{< ref "versioning.md#package-versioning" >}} ) for more information.

API versioning is just as important for D-Bus APIs and GSettings schemas (if
they are likely to change) as for C APIs. See the [documentation on D-Bus API
versioning](http://dbus.freedesktop.org/doc/dbus-api-design.html#api-versioning)
for details.

For GIR APIs, their stability typically follows the C API stability, as they
are generated from the C API. One complexity is that their stability
additionally depends on the version of gobject-introspection used in generating
the GIR, but recent versions have not changed much so this is not a major
concern.

# Checking for API users

If you are carrying out an API or ABI break, it can be useful to find
everything that depends or build-depends on a particular library. One
convenient way to do this is by using the `grep-dctrl` command.

For example, a developer considering an API or ABI break in libostree might use
commands like this to determine which packages they need to inspect more
closely:

    $ grep-dctrl -s Filename -F Pre-Depends,Depends,Recommends libostree /var/lib/apt/lists/*_Packages
    Filename: pool/target/f/flatpak/flatpak_1.2.5-0+deb10u3co2bv2021.0b1_amd64.deb
    Filename: pool/target/f/flatpak/flatpak-tests_1.2.5-0+deb10u3co2bv2021.0b1_amd64.deb
    Filename: pool/target/f/flatpak/libflatpak-dev_1.2.5-0+deb10u3co2bv2021.0b1_amd64.deb
    Filename: pool/target/f/flatpak/libflatpak0_1.2.5-0+deb10u3co2bv2021.0b1_amd64.deb
    Filename: pool/sdk/d/debos/debos_1.0.0+git20201203.e939090-4apertis1~v2021bv2021.0b1_amd64.deb
    Filename: pool/target/f/flatpak/flatpak_1.2.5-0+deb10u3co1bv2021.0b1_amd64.deb
    Filename: pool/target/f/flatpak/flatpak-tests_1.2.5-0+deb10u3co1bv2021.0b1_amd64.deb
    Filename: pool/target/f/flatpak/libflatpak-dev_1.2.5-0+deb10u3co1bv2021.0b1_amd64.deb
    Filename: pool/target/f/flatpak/libflatpak0_1.2.5-0+deb10u3co1bv2021.0b1_amd64.deb
    Filename: pool/sdk/d/debos/debos_1.0.0+git20201203.e939090-1co1bv2021.0b2_amd64.deb
    Filename: pool/sdk/f/flatpak-builder/flatpak-builder_1.0.5-1co1bv2021.0b2_amd64.deb
    Filename: pool/sdk/g/golang-github-sjoerdsimons-ostree-go/golang-github-sjoerdsimons-ostree-go-dev_0.0~git20201014.8fae757-2~bpo10+1bv2021.0b3_all.deb
    Filename: pool/target/a/apertis-update-manager/apertis-update-manager_0.2020.23bv2021.0b2_amd64.deb
    Filename: pool/target/f/flatpak/flatpak_1.2.5-0+deb10u1co1bv2021preb1_amd64.deb
    Filename: pool/target/f/flatpak/flatpak-tests_1.2.5-0+deb10u1co1bv2021preb1_amd64.deb
    Filename: pool/target/o/ostree/gir1.2-ostree-1.0_2020.7-1co1bv2021preb2_amd64.deb
    Filename: pool/target/f/flatpak/libflatpak-dev_1.2.5-0+deb10u1co1bv2021preb1_amd64.deb
    Filename: pool/target/f/flatpak/libflatpak0_1.2.5-0+deb10u1co1bv2021preb1_amd64.deb
    Filename: pool/target/o/ostree/libostree-1-1-dbgsym_2020.7-1co1bv2021preb2_amd64.deb
    Filename: pool/target/o/ostree/libostree-dev_2020.7-1co1bv2021preb2_amd64.deb
    Filename: pool/target/o/ostree/ostree_2020.7-1co1bv2021preb2_amd64.deb
    Filename: pool/target/o/ostree/ostree-boot_2020.7-1co1bv2021preb2_amd64.deb
    Filename: pool/target/o/ostree/ostree-tests_2020.7-1co1bv2021preb2_amd64.deb
    Filename: pool/target/r/ribchester/ribchester-core_0.2020.3bv2021dev2b1_amd64.deb
    Filename: pool/target/r/ribchester/ribchester-dev_0.2020.3bv2021dev2b1_amd64.deb
    Filename: pool/target/r/ribchester/ribchester-full_0.2020.3bv2021dev2b1_amd64.deb
    user@apertis:~$ grep-dctrl -s Directory -F Build-Depends,Build-Depends-Arch,Build-Depends-Indep libostree /var/lib/apt/lists/*_Sources
    Directory: pool/target/f/flatpak
    Directory: pool/target/f/flatpak
    Directory: pool/sdk/f/flatpak-builder
    Directory: pool/sdk/g/golang-github-sjoerdsimons-ostree-go
    Directory: pool/target/a/apertis-update-manager
    Directory: pool/target/f/flatpak
    Directory: pool/target/r/ribchester

# External links

- [Wikipedia page on APIs](http://en.wikipedia.org/wiki/Application_programming_interface)
- [Wikipedia page on ABIs](http://en.wikipedia.org/wiki/Application_binary_interface)
- [D-Bus API versioning documentation](http://dbus.freedesktop.org/doc/dbus-api-design.html#api-versioning)
